@@ -106,7 +106,7 @@ gst_rtp_vp8_depay_process (GstRTPBaseDepayload * depay, GstBuffer * buf)
   GstRtpVP8Depay *self = GST_RTP_VP8_DEPAY (depay);
   GstBuffer *payload;
   guint8 *data;
-  guint offset;
+  guint hdrsize;
   guint size;
   GstRTPBuffer rtpbuffer = GST_RTP_BUFFER_INIT;
 
@@ -134,42 +134,54 @@ gst_rtp_vp8_depay_process (GstRTPBaseDepayload * depay, GstBuffer * buf)
     self->started = TRUE;
   }
 
-  offset = 1;
+  hdrsize = 1;
   /* Check X optional header */
   if ((data[0] & 0x80) != 0) {
-    offset++;
+    hdrsize++;
     /* Check I optional header */
     if ((data[1] & 0x80) != 0) {
-      offset++;
-      if (G_UNLIKELY (offset + 2 >= size))
+      if (G_UNLIKELY (size < 3))
         goto too_small;
+      hdrsize++;
       /* Check for 16 bits PictureID */
       if ((data[2] & 0x80) != 0)
-        offset++;
+        hdrsize++;
     }
     /* Check L optional header */
     if ((data[1] & 0x40) != 0)
-      offset++;
+      hdrsize++;
     /* Check T or K optional headers */
     if ((data[1] & 0x20) != 0 || (data[1] & 0x10) != 0)
-      offset++;
+      hdrsize++;
   }
+  GST_DEBUG_OBJECT (depay, "hdrsize %u, size %u", hdrsize, size);
 
-  if (G_UNLIKELY (offset >= size))
+  if (G_UNLIKELY (hdrsize >= size))
     goto too_small;
 
-  payload = gst_rtp_buffer_get_payload_subbuffer (&rtpbuffer, offset, -1);
+  payload = gst_rtp_buffer_get_payload_subbuffer (&rtpbuffer, hdrsize, -1);
   gst_adapter_push (self->adapter, payload);
 
   /* Marker indicates that it was the last rtp packet for this frame */
   if (gst_rtp_buffer_get_marker (&rtpbuffer)) {
     GstBuffer *out;
+    guint8 flag0;
+
+    gst_adapter_copy (self->adapter, &flag0, 0, 1);
 
     out = gst_adapter_take_buffer (self->adapter,
         gst_adapter_available (self->adapter));
 
     self->started = FALSE;
     gst_rtp_buffer_unmap (&rtpbuffer);
+
+    /* mark keyframes */
+    out = gst_buffer_make_writable (out);
+    if ((flag0 & 0x01))
+      GST_BUFFER_FLAG_SET (out, GST_BUFFER_FLAG_DELTA_UNIT);
+    else
+      GST_BUFFER_FLAG_UNSET (out, GST_BUFFER_FLAG_DELTA_UNIT);
+
     return out;
   }
 
