@@ -21,11 +21,6 @@
  */
 
 #include <gst/check/gstcheck.h>
-#include <gst/check/gsttestclock.h>
-#include <gst/check/gstharness.h>
-
-#include <gst/rtp/gstrtpbuffer.h>
-#include <gst/rtp/gstrtcpbuffer.h>
 
 GST_START_TEST (test_pads)
 {
@@ -34,7 +29,7 @@ GST_START_TEST (test_pads)
 
   element = gst_element_factory_make ("rtpsession", NULL);
 
-  pad = gst_element_request_pad_simple (element, "recv_rtcp_sink");
+  pad = gst_element_get_request_pad (element, "recv_rtcp_sink");
   gst_object_unref (pad);
   gst_object_unref (element);
 }
@@ -52,7 +47,7 @@ GST_START_TEST (test_cleanup_send)
 
   while (count--) {
     /* request session 0 */
-    rtp_sink = gst_element_request_pad_simple (rtpbin, "send_rtp_sink_0");
+    rtp_sink = gst_element_get_request_pad (rtpbin, "send_rtp_sink_0");
     fail_unless (rtp_sink != NULL);
     ASSERT_OBJECT_REFCOUNT (rtp_sink, "rtp_sink", 2);
 
@@ -67,7 +62,7 @@ GST_START_TEST (test_cleanup_send)
     g_object_unref (session);
 
     /* get the send RTCP pad too */
-    rtcp_src = gst_element_request_pad_simple (rtpbin, "send_rtcp_src_0");
+    rtcp_src = gst_element_get_request_pad (rtpbin, "send_rtcp_src_0");
     fail_unless (rtcp_src != NULL);
     ASSERT_OBJECT_REFCOUNT (rtcp_src, "rtcp_src", 2);
 
@@ -116,7 +111,6 @@ typedef struct
   GCond cond;
   GstPad *sinkpad;
   GList *pads;
-  GstCaps *caps;
 } CleanupData;
 
 static void
@@ -127,7 +121,6 @@ init_data (CleanupData * data)
   g_mutex_init (&data->lock);
   g_cond_init (&data->cond);
   data->pads = NULL;
-  data->caps = NULL;
 }
 
 static void
@@ -137,8 +130,6 @@ clean_data (CleanupData * data)
   g_list_free (data->pads);
   g_mutex_clear (&data->lock);
   g_cond_clear (&data->cond);
-  if (data->caps)
-    gst_caps_unref (data->caps);
 }
 
 static guint8 rtp_packet[] = { 0x80, 0x60, 0x94, 0xbc, 0x8f, 0x37, 0x4e, 0xb8,
@@ -151,19 +142,20 @@ static GstFlowReturn
 chain_rtp_packet (GstPad * pad, CleanupData * data)
 {
   GstFlowReturn res;
+  static GstCaps *caps = NULL;
   GstSegment segment;
   GstBuffer *buffer;
   GstMapInfo map;
 
-  if (data->caps == NULL) {
-    data->caps = gst_caps_from_string ("application/x-rtp,"
+  if (caps == NULL) {
+    caps = gst_caps_from_string ("application/x-rtp,"
         "media=(string)audio, clock-rate=(int)44100, "
         "encoding-name=(string)L16, encoding-params=(string)1, channels=(int)1");
     data->seqnum = 0;
   }
 
   gst_pad_send_event (pad, gst_event_new_stream_start (GST_OBJECT_NAME (pad)));
-  gst_pad_send_event (pad, gst_event_new_caps (data->caps));
+  gst_pad_send_event (pad, gst_event_new_caps (caps));
   gst_segment_init (&segment, GST_FORMAT_TIME);
   gst_pad_send_event (pad, gst_event_new_segment (&segment));
 
@@ -177,8 +169,6 @@ chain_rtp_packet (GstPad * pad, CleanupData * data)
   data->seqnum++;
   gst_buffer_unmap (buffer, &map);
 
-  GST_BUFFER_DTS (buffer) = 0;
-
   res = gst_pad_chain (pad, buffer);
 
   return res;
@@ -272,7 +262,7 @@ GST_START_TEST (test_cleanup_recv)
 
   while (count--) {
     /* request session 0 */
-    rtp_sink = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_0");
+    rtp_sink = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_0");
     fail_unless (rtp_sink != NULL);
     ASSERT_OBJECT_REFCOUNT (rtp_sink, "rtp_sink", 2);
 
@@ -343,7 +333,7 @@ GST_START_TEST (test_cleanup_recv2)
   fail_unless (ret == GST_STATE_CHANGE_SUCCESS);
 
   /* request session 0 */
-  rtp_sink = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_0");
+  rtp_sink = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_0");
   fail_unless (rtp_sink != NULL);
   ASSERT_OBJECT_REFCOUNT (rtp_sink, "rtp_sink", 2);
 
@@ -409,17 +399,17 @@ GST_START_TEST (test_request_pad_by_template_name)
   GstPad *rtp_sink1, *rtp_sink2, *rtp_sink3;
 
   rtpbin = gst_element_factory_make ("rtpbin", "rtpbin");
-  rtp_sink1 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_%u");
+  rtp_sink1 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_%u");
   fail_unless (rtp_sink1 != NULL);
   fail_unless_equals_string (GST_PAD_NAME (rtp_sink1), "recv_rtp_sink_0");
   ASSERT_OBJECT_REFCOUNT (rtp_sink1, "rtp_sink1", 2);
 
-  rtp_sink2 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_%u");
+  rtp_sink2 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_%u");
   fail_unless (rtp_sink2 != NULL);
   fail_unless_equals_string (GST_PAD_NAME (rtp_sink2), "recv_rtp_sink_1");
   ASSERT_OBJECT_REFCOUNT (rtp_sink2, "rtp_sink2", 2);
 
-  rtp_sink3 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_%u");
+  rtp_sink3 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_%u");
   fail_unless (rtp_sink3 != NULL);
   fail_unless_equals_string (GST_PAD_NAME (rtp_sink3), "recv_rtp_sink_2");
   ASSERT_OBJECT_REFCOUNT (rtp_sink3, "rtp_sink3", 2);
@@ -487,7 +477,7 @@ GST_START_TEST (test_encoder)
   id = g_signal_connect (rtpbin, "request-rtp-encoder", (GCallback) encoder_cb,
       bin);
 
-  rtp_sink1 = gst_element_request_pad_simple (rtpbin, "send_rtp_sink_2");
+  rtp_sink1 = gst_element_get_request_pad (rtpbin, "send_rtp_sink_2");
   fail_unless (rtp_sink1 != NULL);
   fail_unless_equals_string (GST_PAD_NAME (rtp_sink1), "send_rtp_sink_2");
   ASSERT_OBJECT_REFCOUNT (rtp_sink1, "rtp_sink1", 2);
@@ -497,7 +487,7 @@ GST_START_TEST (test_encoder)
   id = g_signal_connect (rtpbin, "request-rtp-encoder", (GCallback) encoder_cb2,
       bin);
 
-  rtp_sink2 = gst_element_request_pad_simple (rtpbin, "send_rtp_sink_3");
+  rtp_sink2 = gst_element_get_request_pad (rtpbin, "send_rtp_sink_3");
   fail_unless (rtp_sink2 != NULL);
 
   /* remove the session */
@@ -547,12 +537,12 @@ GST_START_TEST (test_decoder)
   id = g_signal_connect (rtpbin, "request-rtp-decoder", (GCallback) decoder_cb,
       NULL);
 
-  rtp_sink1 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_2");
+  rtp_sink1 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_2");
   fail_unless (rtp_sink1 != NULL);
   fail_unless_equals_string (GST_PAD_NAME (rtp_sink1), "recv_rtp_sink_2");
   ASSERT_OBJECT_REFCOUNT (rtp_sink1, "rtp_sink1", 2);
 
-  rtp_sink2 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_3");
+  rtp_sink2 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_3");
   fail_unless (rtp_sink2 != NULL);
 
   g_signal_handler_disconnect (rtpbin, id);
@@ -579,7 +569,7 @@ aux_sender_cb (GstElement * rtpbin, guint sessid, gpointer user_data)
   GstElement *bin;
   GstPad *srcpad, *sinkpad;
 
-  bin = (GstElement *) user_data;
+  bin = gst_bin_new (NULL);
 
   GST_DEBUG ("making AUX sender");
   sinkpad = gst_ghost_pad_new_no_target ("sink_2", GST_PAD_SINK);
@@ -600,16 +590,13 @@ GST_START_TEST (test_aux_sender)
   GstElement *rtpbin;
   GstPad *rtp_sink1, *rtp_src, *rtcp_src;
   gulong id;
-  GstElement *aux_sender = gst_object_ref_sink (gst_bin_new ("aux-sender"));
-
-  gst_object_ref (aux_sender);
 
   rtpbin = gst_element_factory_make ("rtpbin", "rtpbin");
 
   id = g_signal_connect (rtpbin, "request-aux-sender",
-      (GCallback) aux_sender_cb, aux_sender);
+      (GCallback) aux_sender_cb, NULL);
 
-  rtp_sink1 = gst_element_request_pad_simple (rtpbin, "send_rtp_sink_2");
+  rtp_sink1 = gst_element_get_request_pad (rtpbin, "send_rtp_sink_2");
   fail_unless (rtp_sink1 != NULL);
   fail_unless_equals_string (GST_PAD_NAME (rtp_sink1), "send_rtp_sink_2");
   ASSERT_OBJECT_REFCOUNT (rtp_sink1, "rtp_sink1", 2);
@@ -624,7 +611,7 @@ GST_START_TEST (test_aux_sender)
   fail_unless (rtp_src != NULL);
   gst_object_unref (rtp_src);
 
-  rtcp_src = gst_element_request_pad_simple (rtpbin, "send_rtcp_src_1");
+  rtcp_src = gst_element_get_request_pad (rtpbin, "send_rtcp_src_1");
   fail_unless (rtcp_src != NULL);
   gst_element_release_request_pad (rtpbin, rtcp_src);
   gst_object_unref (rtcp_src);
@@ -637,15 +624,6 @@ GST_START_TEST (test_aux_sender)
   gst_element_release_request_pad (rtpbin, rtp_sink1);
   gst_object_unref (rtp_sink1);
 
-  /* We have sinked the initial reference before returning it
-   * in the request callback, the ref count should now be 1 because
-   * the return of the signal is transfer full, and rtpbin should
-   * have released that reference by now, but we had taken an
-   * extra reference to perform this check
-   */
-  ASSERT_OBJECT_REFCOUNT (aux_sender, "aux-sender", 1);
-
-  gst_object_unref (aux_sender);
   gst_object_unref (rtpbin);
 }
 
@@ -684,15 +662,15 @@ GST_START_TEST (test_aux_receiver)
   id = g_signal_connect (rtpbin, "request-aux-receiver",
       (GCallback) aux_receiver_cb, NULL);
 
-  rtp_sink1 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_2");
+  rtp_sink1 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_2");
   fail_unless (rtp_sink1 != NULL);
 
-  rtp_sink2 = gst_element_request_pad_simple (rtpbin, "recv_rtp_sink_1");
+  rtp_sink2 = gst_element_get_request_pad (rtpbin, "recv_rtp_sink_1");
   fail_unless (rtp_sink2 != NULL);
 
   g_signal_handler_disconnect (rtpbin, id);
 
-  rtcp_sink = gst_element_request_pad_simple (rtpbin, "recv_rtcp_sink_1");
+  rtcp_sink = gst_element_get_request_pad (rtpbin, "recv_rtcp_sink_1");
   fail_unless (rtcp_sink != NULL);
   gst_element_release_request_pad (rtpbin, rtcp_sink);
   gst_object_unref (rtcp_sink);
@@ -708,269 +686,8 @@ GST_START_TEST (test_aux_receiver)
 
 GST_END_TEST;
 
-GST_START_TEST (test_sender_eos)
-{
-  GstElement *rtpsession;
-  GstBuffer *rtp_buffer;
-  GstBuffer *rtcp_buffer;
-  GstRTPBuffer rtpbuf = GST_RTP_BUFFER_INIT;
-  GstRTCPBuffer rtcpbuf = GST_RTCP_BUFFER_INIT;
-  GstRTCPPacket rtcppacket;
-  static GstStaticPadTemplate recv_tmpl =
-      GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
-      GST_STATIC_CAPS ("ANY"));
-  GstPad *send_rtp_sink;
-  GstPad *recv_rtcp_sink;
-  GstCaps *caps;
-  GstSegment segment;
-  GstPad *rtp_sink, *rtcp_sink;
-  GstClock *clock;
-  GstTestClock *tclock;
-  GstStructure *s;
-  guint ssrc = 1;
-  guint32 ssrc_in, packet_count, octet_count;
-  gboolean got_bye = FALSE;
-
-  clock = gst_test_clock_new ();
-  gst_system_clock_set_default (clock);
-  tclock = GST_TEST_CLOCK (clock);
-  gst_test_clock_set_time (tclock, 0);
-
-  rtpsession = gst_element_factory_make ("rtpsession", NULL);
-  send_rtp_sink = gst_element_request_pad_simple (rtpsession, "send_rtp_sink");
-  recv_rtcp_sink =
-      gst_element_request_pad_simple (rtpsession, "recv_rtcp_sink");
-
-
-  rtp_sink = gst_check_setup_sink_pad_by_name (rtpsession, &recv_tmpl,
-      "send_rtp_src");
-  rtcp_sink = gst_check_setup_sink_pad_by_name (rtpsession, &recv_tmpl,
-      "send_rtcp_src");
-
-  gst_pad_set_active (rtp_sink, TRUE);
-  gst_pad_set_active (rtcp_sink, TRUE);
-
-  gst_element_set_state (rtpsession, GST_STATE_PLAYING);
-
-  /* Send initial events */
-
-  gst_segment_init (&segment, GST_FORMAT_TIME);
-  fail_unless (gst_pad_send_event (send_rtp_sink,
-          gst_event_new_stream_start ("id")));
-  fail_unless (gst_pad_send_event (send_rtp_sink,
-          gst_event_new_segment (&segment)));
-
-  fail_unless (gst_pad_send_event (recv_rtcp_sink,
-          gst_event_new_stream_start ("id")));
-  fail_unless (gst_pad_send_event (recv_rtcp_sink,
-          gst_event_new_segment (&segment)));
-
-  /* Get the suggested SSRC from the rtpsession */
-
-  caps = gst_pad_query_caps (send_rtp_sink, NULL);
-  s = gst_caps_get_structure (caps, 0);
-  gst_structure_get (s, "ssrc", G_TYPE_UINT, &ssrc, NULL);
-  gst_caps_unref (caps);
-
-  /* Send a RTP packet */
-
-  rtp_buffer = gst_rtp_buffer_new_allocate (10, 0, 0);
-  gst_rtp_buffer_map (rtp_buffer, GST_MAP_READWRITE, &rtpbuf);
-  gst_rtp_buffer_set_ssrc (&rtpbuf, 1);
-  gst_rtp_buffer_set_seq (&rtpbuf, 0);
-  gst_rtp_buffer_unmap (&rtpbuf);
-
-  fail_unless (gst_pad_chain (send_rtp_sink, rtp_buffer) == GST_FLOW_OK);
-
-  /* Make sure it went through */
-  fail_unless_equals_int (g_list_length (buffers), 1);
-  fail_unless_equals_pointer (buffers->data, rtp_buffer);
-  gst_check_drop_buffers ();
-
-  /* Advance time and send a packet to prevent source sender timeout */
-  gst_test_clock_set_time (tclock, 1 * GST_SECOND);
-
-  /* Just send a send packet to prevent timeout */
-  rtp_buffer = gst_rtp_buffer_new_allocate (10, 0, 0);
-  gst_rtp_buffer_map (rtp_buffer, GST_MAP_READWRITE, &rtpbuf);
-  gst_rtp_buffer_set_ssrc (&rtpbuf, 1);
-  gst_rtp_buffer_set_seq (&rtpbuf, 1);
-  gst_rtp_buffer_set_timestamp (&rtpbuf, 10);
-  gst_rtp_buffer_unmap (&rtpbuf);
-
-  fail_unless (gst_pad_chain (send_rtp_sink, rtp_buffer) == GST_FLOW_OK);
-
-  /* Make sure it went through */
-  fail_unless_equals_int (g_list_length (buffers), 1);
-  fail_unless_equals_pointer (buffers->data, rtp_buffer);
-  gst_check_drop_buffers ();
-
-  /* Advance clock twice and we should have one RTCP packet at least */
-  gst_test_clock_crank (tclock);
-  gst_test_clock_crank (tclock);
-
-  g_mutex_lock (&check_mutex);
-  while (buffers == NULL)
-    g_cond_wait (&check_cond, &check_mutex);
-
-  fail_unless (gst_rtcp_buffer_map (buffers->data, GST_MAP_READ, &rtcpbuf));
-
-  fail_unless (gst_rtcp_buffer_get_first_packet (&rtcpbuf, &rtcppacket));
-
-  fail_unless_equals_int (gst_rtcp_packet_get_type (&rtcppacket),
-      GST_RTCP_TYPE_SR);
-  gst_rtcp_packet_sr_get_sender_info (&rtcppacket, &ssrc_in, NULL, NULL,
-      &packet_count, &octet_count);
-  fail_unless_equals_int (packet_count, 2);
-  fail_unless_equals_int (octet_count, 20);
-
-  fail_unless (gst_rtcp_packet_move_to_next (&rtcppacket));
-  fail_unless_equals_int (gst_rtcp_packet_get_type (&rtcppacket),
-      GST_RTCP_TYPE_SDES);
-
-  gst_rtcp_buffer_unmap (&rtcpbuf);
-  gst_check_drop_buffers ();
-
-  g_mutex_unlock (&check_mutex);
-
-
-  /* Create and send a valid RTCP reply packet */
-  rtcp_buffer = gst_rtcp_buffer_new (1500);
-  gst_rtcp_buffer_map (rtcp_buffer, GST_MAP_READWRITE, &rtcpbuf);
-  gst_rtcp_buffer_add_packet (&rtcpbuf, GST_RTCP_TYPE_RR, &rtcppacket);
-  gst_rtcp_packet_rr_set_ssrc (&rtcppacket, ssrc + 1);
-  gst_rtcp_packet_add_rb (&rtcppacket, ssrc, 0, 0, 0, 0, 0, 0);
-  gst_rtcp_buffer_add_packet (&rtcpbuf, GST_RTCP_TYPE_SDES, &rtcppacket);
-  gst_rtcp_packet_sdes_add_item (&rtcppacket, ssrc + 1);
-  gst_rtcp_packet_sdes_add_entry (&rtcppacket, GST_RTCP_SDES_CNAME, 3,
-      (guint8 *) "a@a");
-  gst_rtcp_packet_sdes_add_entry (&rtcppacket, GST_RTCP_SDES_NAME, 2,
-      (guint8 *) "aa");
-  gst_rtcp_packet_sdes_add_entry (&rtcppacket, GST_RTCP_SDES_END, 0,
-      (guint8 *) "");
-  gst_rtcp_buffer_unmap (&rtcpbuf);
-  fail_unless (gst_pad_chain (recv_rtcp_sink, rtcp_buffer) == GST_FLOW_OK);
-
-
-  /* Send a EOS to trigger sending a BYE message */
-  fail_unless (gst_pad_send_event (send_rtp_sink, gst_event_new_eos ()));
-
-  /* Crank to process EOS and wait for BYE */
-  for (;;) {
-    gst_test_clock_crank (tclock);
-    g_mutex_lock (&check_mutex);
-    while (buffers == NULL)
-      g_cond_wait (&check_cond, &check_mutex);
-
-    fail_unless (gst_rtcp_buffer_map (g_list_last (buffers)->data, GST_MAP_READ,
-            &rtcpbuf));
-    fail_unless (gst_rtcp_buffer_get_first_packet (&rtcpbuf, &rtcppacket));
-
-    while (gst_rtcp_packet_move_to_next (&rtcppacket)) {
-      if (gst_rtcp_packet_get_type (&rtcppacket) == GST_RTCP_TYPE_BYE) {
-        got_bye = TRUE;
-        break;
-      }
-    }
-    g_mutex_unlock (&check_mutex);
-    gst_rtcp_buffer_unmap (&rtcpbuf);
-
-    if (got_bye)
-      break;
-  }
-
-  gst_check_drop_buffers ();
-
-
-  fail_unless (GST_PAD_IS_EOS (rtp_sink));
-  fail_unless (GST_PAD_IS_EOS (rtcp_sink));
-
-  gst_pad_set_active (rtp_sink, FALSE);
-  gst_pad_set_active (rtcp_sink, FALSE);
-
-  gst_check_teardown_pad_by_name (rtpsession, "send_rtp_src");
-  gst_check_teardown_pad_by_name (rtpsession, "send_rtcp_src");
-  gst_element_release_request_pad (rtpsession, send_rtp_sink);
-  gst_object_unref (send_rtp_sink);
-  gst_element_release_request_pad (rtpsession, recv_rtcp_sink);
-  gst_object_unref (recv_rtcp_sink);
-
-  gst_check_teardown_element (rtpsession);
-
-  gst_system_clock_set_default (NULL);
-  gst_object_unref (clock);
-
-}
-
-GST_END_TEST;
-
-static GstBuffer *
-generate_rtp_buffer (GstClockTime ts,
-    guint seqnum, guint32 rtp_ts, guint pt, guint ssrc)
-{
-  GstRTPBuffer rtp = GST_RTP_BUFFER_INIT;
-  GstBuffer *buf = gst_rtp_buffer_new_allocate (0, 0, 0);
-  GST_BUFFER_PTS (buf) = ts;
-  GST_BUFFER_DTS (buf) = ts;
-
-  gst_rtp_buffer_map (buf, GST_MAP_READWRITE, &rtp);
-  gst_rtp_buffer_set_payload_type (&rtp, pt);
-  gst_rtp_buffer_set_seq (&rtp, seqnum);
-  gst_rtp_buffer_set_timestamp (&rtp, rtp_ts);
-  gst_rtp_buffer_set_ssrc (&rtp, ssrc);
-
-  gst_rtp_buffer_unmap (&rtp);
-
-  return buf;
-}
-
-static GstCaps *
-_request_pt_map (G_GNUC_UNUSED GstElement * rtpbin,
-    G_GNUC_UNUSED guint session_id, G_GNUC_UNUSED guint pt,
-    const GstCaps * caps)
-{
-  return gst_caps_copy (caps);
-}
-
-static void
-_pad_added (G_GNUC_UNUSED GstElement * rtpbin, GstPad * pad, GstHarness * h)
-{
-  gst_harness_add_element_src_pad (h, pad);
-}
-
-GST_START_TEST (test_quick_shutdown)
-{
-  guint r;
-
-  for (r = 0; r < 1000; r++) {
-    guint i;
-    GstHarness *h = gst_harness_new_with_padnames ("rtpbin",
-        "recv_rtp_sink_0", NULL);
-    GstCaps *caps = gst_caps_new_simple ("application/x-rtp",
-        "clock-rate", G_TYPE_INT, 8000,
-        "payload", G_TYPE_INT, 100, NULL);
-
-    g_signal_connect (h->element, "request-pt-map",
-        G_CALLBACK (_request_pt_map), caps);
-    g_signal_connect (h->element, "pad-added", G_CALLBACK (_pad_added), h);
-
-    gst_harness_set_src_caps (h, gst_caps_copy (caps));
-
-    for (i = 0; i < 50; i++) {
-      gst_harness_push (h,
-          generate_rtp_buffer (i * GST_MSECOND * 20, i, i * 160, 100, 1234));
-    }
-    gst_harness_crank_single_clock_wait (h);
-
-    gst_caps_unref (caps);
-    gst_harness_teardown (h);
-  }
-}
-
-GST_END_TEST;
-
 static Suite *
-rtpbin_suite (void)
+gstrtpbin_suite (void)
 {
   Suite *s = suite_create ("rtpbin");
   TCase *tc_chain = tcase_create ("general");
@@ -985,10 +702,23 @@ rtpbin_suite (void)
   tcase_add_test (tc_chain, test_decoder);
   tcase_add_test (tc_chain, test_aux_sender);
   tcase_add_test (tc_chain, test_aux_receiver);
-  tcase_add_test (tc_chain, test_sender_eos);
-  tcase_add_test (tc_chain, test_quick_shutdown);
 
   return s;
 }
 
-GST_CHECK_MAIN (rtpbin);
+int
+main (int argc, char **argv)
+{
+  int nf;
+
+  Suite *s = gstrtpbin_suite ();
+  SRunner *sr = srunner_create (s);
+
+  gst_check_init (&argc, &argv);
+
+  srunner_run_all (sr, CK_NORMAL);
+  nf = srunner_ntests_failed (sr);
+  srunner_free (sr);
+
+  return nf;
+}
